use serde::{Deserialize, Serialize};
use url::Url;
use std::sync::Arc;
use std::collections::HashMap;

use tokio::sync::RwLock;

use handlebars::Handlebars;
use webauthn_rs::prelude::{Uuid, PasskeyRegistration, Passkey};
use webauthn_rs::{Webauthn, WebauthnBuilder};

pub type DbField<T> = Arc<RwLock<T>>;

#[derive(Clone)]
pub struct Db {
    pub user_keys: DbField<HashMap<Uuid, Passkey>>,
    pub user_uuid_object: DbField<HashMap<Uuid, User>>,
}

#[derive(Clone)]
pub struct TempSession {
    pub user_registrations: Arc<RwLock<HashMap<User,PasskeyRegistration>>>
}

#[derive(Serialize, Deserialize, Clone, PartialEq, Eq, Hash)]
pub struct User {
    pub uuid: Uuid,
    pub user_name: String,
    pub display_name: String,
}

#[derive(Clone)]
pub struct AppState<'a>{
    pub webauthn: Arc<Webauthn>,
    pub db: Db,
    pub hbs: Arc<Handlebars<'a>>,
    pub session: TempSession
}

impl AppState<'_> {
    pub fn new() -> Self {
        let rp = "localhost";
        let rp_origin = Url::parse("http://localhost:8000").expect("Invalid URL");
        let builder = WebauthnBuilder::new(rp, &rp_origin).expect("Invalid configuration");
        let builder = builder.rp_name("LocalHost");
        let webauthn = Arc::new(builder.build().expect("Invalid configuration"));
        let db: Db = Db {
            user_keys: Arc::new(RwLock::new(HashMap::new())),
            user_uuid_object: Arc::new(RwLock::new(HashMap::new()))

        };
        let hbs = Arc::new(crate::templates::new().unwrap());
        let session: TempSession = TempSession {
            user_registrations: Arc::new(RwLock::new(HashMap::new()))
        };

        AppState {
            webauthn,
            db,
            hbs,
            session
        }
    }
}
